package actors;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.StandardOpenOption;
import java.util.ArrayList;

import model.Model;
import model.UserStory;
import plugin.Controller;
import plugin.LogWriter;
import plugin.serialization.JsonManager;

public class ActorManager
{
	public static File actorsFile;
	public static ArrayList<Actor> actorsList;
	public static int lastId = 0;
	

	public static String ACTOR_NONE_NAME = "No Actor";
	public static int ACTOR_NONE_ID = 0;
	
	//When Actors are about to be saved, all actors assigned with this ID are updated with a new one.
	public static int ACTOR_NEW_ID = -1;
	
	public static void addActor(int id, String actorName)
	{
		if (actorsFile==null)
		{
			////LogWriter.info("ActorManager: addActor: Actors File Is Null (1st pass)! Attempting to find it");
			actorsFile = Controller.getInstance().getFile(Controller.PATH_FILE_ACTORS);
		}
		
		if (actorsFile!=null)
		{
			if (!actorsFile.exists())
			{
				////LogWriter.info("ActorManager: addActor: actors.txt does not exist!");
			}
			else
			{
				LogWriter.info("ActorManager: addActor: Reading from " + actorsFile.getAbsolutePath().toString());
				//LogWriter.write("ActorManager: addActor: Reading from " + actorsFile.getAbsolutePath().toString() ,LogWriter.SEVERITY_INFO);
				String lineToWrite = id + "," + actorName;
				
				String line = lineToWrite + "\n";
				try
				{
					Files.write(actorsFile.toPath(),line.getBytes(),StandardOpenOption.APPEND);
				}
				catch (IOException e)
				{
					LogWriter.write("ActorManager: addActor: IOException occured during File Write",LogWriter.SEVERITY_MEDIUM);
				}
			}
		}
		else
		{
			////LogWriter.info("ActorManager: addActor: Actors File Is Null (2nd pass)! Aborting");
		}
	}
	
	public static void updateActorsList()
	{
		actorsList = new ArrayList<Actor>();
		ArrayList<String> actorFileLines = new ArrayList<String>();
		
		if (actorsFile==null)
		{
			////LogWriter.info("ActorManager: addActor: Actors File Is Null (1st pass)! Attempting to find it");
			actorsFile = Controller.getInstance().getFile(Controller.PATH_FILE_ACTORS);
		}
		
		if (actorsFile!=null)
		{
			if (!actorsFile.exists())
			{
				////LogWriter.info("ActorManager: getActors: actors.txt does not exist!");
			}
			else
			{
				try
				{
					actorFileLines = (ArrayList<String>) Files.readAllLines(actorsFile.toPath(), StandardCharsets.UTF_8);
				}
				catch (IOException e)
				{
					//LogWriter.info("ActorManager: getActors: IOException occured during File Read");
				}
			}
		}
		else
		{
			//LogWriter.info("ActorManager: getActors: Actors File Is Null (2nd pass)! Aborting");
		}
		
		for (String line : actorFileLines)
		{
			actorsList.add(new Actor(line));
		}
		
		//Update last id
		try
		{
			lastId = actorsList.get(actorsList.size()-1).getId();
		}
		catch (ArrayIndexOutOfBoundsException e)
		{
			lastId = 0;
		}
	}
	
	//Returns ALL Actors (including ACTOR_NONE)
	public static ArrayList<String> getActorNames()
	{
		ArrayList<String> actorNamesList = new ArrayList<String>();
		updateActorsList();
		
		for (Actor actor : actorsList)
		{
			actorNamesList.add(actor.getName());
		}
		return actorNamesList;
	}
	
	public static String[] getActorNamesArray()
	{
		ArrayList<String> actorNamesList = getActorNames();
		
		String[] array = new String[actorNamesList.size()];
		for (int i = 0; i<actorNamesList.size(); i++)
		{
			array[i] = actorNamesList.get(i);
		}
		return array;
	}
	
	//Returns all Actors excluding ACTOR_NONE
	public static ArrayList<Actor> getEditableActors()
	{
		updateActorsList();
		ArrayList<Actor> editableActorsList = actorsList;
		
		//Remove ACTOR_NONE
		if (editableActorsList.get(0).getName().equals(ACTOR_NONE_NAME))
		{
			editableActorsList.remove(editableActorsList.get(0));
		}
		else
		{
			//LogWriter.info("ActorManager: getEditableActors: Couldn't find ACTOR_NONE");
		}
		return editableActorsList;
	}
	
	public static ArrayList<Actor> getAllActors()
	{
		updateActorsList();
		return actorsList;
	}
	
	public static void replace(int id, String newName)
	{
		for (Actor actor : actorsList)
		{
			if (actor.getId() == id)
			{
				actor.setName(newName);
				return;
			}
		}
	}
	
	public static Actor getActorById(int id)
	{
		updateActorsList();
		for (Actor actor : actorsList)
		{
			if (actor.getId() == id)
			{
				return actor;
			}
		}
		return new Actor(-2, "No Actor Found With This ID");
	}
	
	public static void resetUSWithDeletedActors(ArrayList<Integer> deletedActorIDs)
	{
		//For every US
		for (Model m : Controller.getDatabase().getAllUS())
		{
			if (m instanceof UserStory)
			{
				UserStory us = (UserStory)m;

				//For every actor
				if (us.getActors()!=null)
				{
					//Check this in reverse so that we don't get java.util.ConcurrentModificationException
					for (int i=us.getActors().size()-1; i> -1; i--)
					{
						//For each of the deleted IDs
						for (int deletedID : deletedActorIDs)
						{
							if (us.getActors().get(i) == deletedID)
							{
								us.getActors().remove(us.getActors().indexOf(deletedID));
							}
						}
					}
				}
				
				if (us.getActors().isEmpty())
				{
					//If there are no Users left after deletions, assign ACTOR_NONE to this US
					us.getActors().add(ACTOR_NONE_ID);
				}
			}
		}
	}
	
	public static void resetUSWithNonExistingActors()
	{
		
	}
	
	//TODO: hopefully this is gone in the future
	//Returns the FIRST occurence of a name. can cause issues if there are many actors with the same name
	public static int getFirstIdFromName(String name)
	{
		updateActorsList();
		for (Actor actor : actorsList)
		{
			if (actor.getName().equals(name))
			{
				return actor.getId();
			}
		}
		return -1;
	}
	
	public static void replaceAllWithList(ArrayList<Actor> actorsL)
	{
		if (actorsFile==null)
		{
			//LogWriter.info("ActorManager: replaceAllWithList: Actors File Is Null (1st pass)! Attempting to find it");
			actorsFile = Controller.getInstance().getFile(Controller.PATH_FILE_ACTORS);
		}
		
		if (actorsFile!=null)
		{
			try
			{
				LogWriter.write("ActorManager: replaceAllWithList: Replacing File " + actorsFile.getAbsolutePath().toString() ,LogWriter.SEVERITY_INFO);
				
				//Exterminate file and make a new one
				actorsFile.delete();
				actorsFile.createNewFile();
				
				
				//Write ACTOR_NONE first
				//Actor none = new Actor(ACTOR_NONE_ID, ACTOR_NONE_NAME);
				//Files.write(actorsFile.toPath(),none.toString().getBytes(),StandardOpenOption.APPEND);
				
				//Write the rest
				if (!actorsL.isEmpty())
				{
					String write = "";
					//Write the rest
					for (Actor actor: actorsL)
					{
						write += actor.toString();
					}
					
					Writer bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(actorsFile.getPath()), "UTF-8"));
					bw.write(write);
					bw.close();
					LogWriter.debug("ActorManager: replaceAllWithList: Finished Saving Actors");
					updateActorsList();
				}
			} 
			catch (IOException e)
			{
				//LogWriter.info("ActorManager: replaceAllWithList: IOException occured during File Write");
			}
		}
		else
		{
			//LogWriter.info("ActorManager: replaceAllWithList: Actors File Is Null (2nd pass)! Aborting");
		}
	}
	
	public static int getIndexById(int id)
	{
		updateActorsList();
		for (int i = 0; i < actorsList.size(); i++)
		{
			if (actorsList.get(i).getId() == id)
			{
				return i;
			}
		}
		
		//ACTOR_NONE
		return 0;
	}
	
	public static int getIndexById(String idStr)
	{
		updateActorsList();
		
		try
		{
			int id = Integer.parseInt(idStr);
			for (int i = 0; i < actorsList.size(); i++)
			{
				if (actorsList.get(i).getId() == id)
				{
					return i;
				}
			}
		}
		catch (NumberFormatException e)
		{
			return 0;
		}
		//ACTOR_NONE
		return 0;
	}
	
	/*public static int index(ArrayList<String> actors, int actorId)
	{
		updateActorsList();
		for (int i = 0 ; i < actorsList.size() ; i ++)
		{
			if (actorsList.get(i).getId() == actorId)
			{
				return i;
			}
		}
		
		//Return ACTOR_NONE if we didn't find anything previously
		return 0;
	}*/
	
	public static ArrayList<String> toListOfStrings(ArrayList<Actor> actors)
	{
		ArrayList<String> retList = new ArrayList<String>();
		for (Actor actor : actors)
		{
			retList.add(actor.toStringNoNewLine());
		}
		return retList;
	}
	
	public static ArrayList<Actor> toListOfActors(ArrayList<String> actorsStr)
	{
		ArrayList<Actor> retList = new ArrayList<Actor>();
		for (String actorStr : actorsStr)
		{
			retList.add(new Actor(actorStr));
		}
		return retList;
	}
	
	public static void freshenIDs(ArrayList<Actor> actors)
	{
		//LogWriter.info("ActorManager: freshenIDs: Entered function");
		//Find Max ID
		if (actors.size()>1)
		{
			int maxID = actors.get(0).getId();
			for (int i = 1 ; i < actors.size() ; i ++)
			{
				if (actors.get(i).getId() > maxID)
				{
					maxID = actors.get(i).getId();
				}
			}
			//LogWriter.info("ActorManager: freshenIDs: Max is " + maxID);
			
			//Freshen IDs
			for (Actor actor : actors)
			{
				if (actor.getId() == ACTOR_NEW_ID )
				{
					maxID++;
					//LogWriter.info("ActorManager: freshenIDs: Giving actor \"" + actor.getName() + "\" the id " + maxID);
					actor.setId(maxID);
				}
			}
		}
		else
		{
			return;
		}
		
	}
}
